// +FHDR------------------------------------------------------------
//                 Copyright (c) 2022 SmartLogic.
//                       ALL RIGHTS RESERVED
// -----------------------------------------------------------------
// Filename      : ucp_handshake.c
// Author        : xianfeng.du
// Created On    : 2022-06-25
// Last Modified :
// -----------------------------------------------------------------
// Description:
//
//
// -FHDR------------------------------------------------------------

#include "typedef.h"
#include "ucp_utility.h"
#include "ucp_printf.h"
//#include "ucp_handshake.h"
//#include "msg_transfer_queue.h"
#include "msg_transfer_mem.h"
#include "pet_sm_mgt.h"

UcpHandshake_t * get_handshake_info(void)
{
    PetSmLocalMgt_t *pPetSmLocalMgt = get_pet_sm_local_mgt();

    return pPetSmLocalMgt->pHandshake;
}

uint32_t get_host_core_mask(void)
{
    uint32_t core_mask = 0;

#ifndef PCIE_BACKHAUL
    /*ARM A72 will call msg_transfer_init on TMAC_BACKHAUL mode*/
    core_mask = (0x1 << NPU_CORE_ID);
#elif (defined(PCIE_WITH_JESD))
    /*ARM A72 & PET RFM SPU0 will call msg_transfer_init on PCIE_WITH_JESD mode*/
    core_mask = (0x1 << NPU_CORE_ID) | (0x01 << PET_RFM_SPU0_CORE_ID);
#else
    /*PET RFM SPU0 will call msg_transfer_init on PCIE_BACKHAUL mode*/
    core_mask = (0x1 << PET_RFM_SPU0_CORE_ID);
#endif

    return core_mask;
}

//host is the core which called msg_transfer_init
void handshake_request_from_host(uint32_t core_mask)
{
    volatile uint32_t request = 0;
    uint32_t core_ready_mask = 0;
    UcpHandshake_t *pHandshake = (UcpHandshake_t *)get_handshake_info();

    while (1)
    {
        for (uint32_t i = 0; i < MAX_NUM_CORE; i++)
        {
            if ((core_mask >> i) & 0x1)
            {
                request = do_read_volatile(&pHandshake->request[i]);
                if (request == (HANDSHKAE_REQ_VALUE + i))
                {
                    UCP_PRINT_EMPTY("Recieved handshake request message from core[0x%08x], value[0x%08x].", i, request);

                    core_ready_mask |= (1 << i);
                }
            }
        }

        if (core_mask == (core_ready_mask & core_mask))
        {
            UCP_PRINT_EMPTY("all cores are ready.");
            break;
        }
    }

    return;
}

//host is the core which called msg_transfer_init
void handshake_response_to_host(uint32_t core_mask)
{
    volatile uint32_t response = 0;
    UcpHandshake_t *pHandshake = (UcpHandshake_t *)get_handshake_info();

    for (uint32_t i = 0; i < MAX_NUM_CORE; i++)
    {
        if ((core_mask >> i) & 0x1)
        {
            response = HANDSHKAE_RESP_VALUE + i;
            do_write(&pHandshake->response[i], response);
            UCP_PRINT_EMPTY("Sent handshake response message to core[0x%08x], value[0x%08x].", i, response);
        }
    }

    return;
}

//msgagent is the core which agented msg_transfer_init,PET_RFM_SPU1_CORE_ID
void handshake_response_to_msgagent(uint32_t core_id)
{
    volatile uint32_t request = 0;
    volatile uint32_t response = HANDSHKAE_RESP_VALUE + core_id;
    UcpHandshake_t *pHandshake = (UcpHandshake_t *)get_handshake_info();

    while (1)
    {
        request = do_read_volatile(&pHandshake->request[core_id]);
        if (request == (HANDSHKAE_REQ_VALUE + core_id))
        {
            UCP_PRINT_EMPTY("Recieved handshake request message from core[0x%08x], value[0x%08x].", core_id, request);

            do_write(&pHandshake->response[core_id], response);
            UCP_PRINT_EMPTY("Sent handshake response message to core[0x%08x], value[0x%08x].", core_id, response);
            break;
        }
    }

    return;
}

//master is the core which controlled the handshake flow
void handshake_master_with_slave(uint32_t core_mask)
{
    uint32_t core_id = get_core_id();
    volatile uint32_t request = 0;
    volatile uint32_t response = 0;
    uint32_t handshake_core_mask = HANDSHKAE_MASK | core_mask;
    uint32_t core_ready_mask = 0;
    UcpHandshake_t *pHandshake = (UcpHandshake_t *)get_handshake_info();

    core_ready_mask = 1 << core_id;
    debug_write(DBG_DDR_COMMON_IDX(core_id, 90), handshake_core_mask);

    //msgagent need to run before other cores.
    handshake_response_to_msgagent(PET_RFM_SPU1_CORE_ID);

    while (1)
    {
        for (uint32_t i = 0; i < MAX_NUM_SPU; i++)
        {
            if ((handshake_core_mask >> i) & 0x1)
            {
                request = do_read_volatile(&pHandshake->request[i]);
                if (request == (HANDSHKAE_REQ_VALUE + i))
                {
                    UCP_PRINT_EMPTY("Recieved handshake request message from core[0x%08x], value[0x%08x].", i, request);

                    response= (HANDSHKAE_RESP_VALUE + i);
                    do_write(&pHandshake->response[i], response);
                    UCP_PRINT_EMPTY("Sent handshake response message to core[0x%08x], value[0x%08x].", i, response);

                    core_ready_mask |= (1 << i);
                }
            }
        }

        if (handshake_core_mask == (core_ready_mask & handshake_core_mask))
        {
            UCP_PRINT_EMPTY("all cores are ready.");
            break;
        }
    }

    return;
}

//master is the core which controlled the handshake flow
void handshake_slave_with_master(void)
{
    uint32_t core_id = get_core_id();
    volatile uint32_t request = HANDSHKAE_REQ_VALUE + core_id;
    volatile uint32_t response = 0;
    UcpHandshake_t *pHandshake = (UcpHandshake_t *)get_handshake_info();

    do_write(&pHandshake->request[core_id], request);
    UCP_PRINT_EMPTY("core[0x%08x] sent handshake request message, value[0x%08x].", core_id, request);

    while (1)
    {
        response = do_read_volatile(&pHandshake->response[core_id]);
        if (response == (HANDSHKAE_RESP_VALUE + core_id))
        {
            UCP_PRINT_EMPTY("core[0x%08x] recieved handshake response message, value[0x%08x].", core_id, response);
            break;
        }
    }

    return;
}


